//source Move.cpp
//Last changed 24.01.2023

#define _DEF_FILES //not redefine the file names included through files.h
#define _DEF_INTERACTION_FUNC//not to redefine the pointer to the intercation functions
#include"Move.h"

//the Move::MakeMove function is defined  in the MakeMove.cpp file
//and its header in the Move.h file.

//Defining the static members
int   Move::nmoved;//number of atoms involved in the move
int   Move::ntypes;//number of atom types in the configuration
int	  Move::npartials;//number of partials (this is needed for the modpart array)
int   Move::moveout;//moveout option indicator
int   Move::ndist_bins;//number of histograms bins for the distances generated by the move (same for each histogram)
int	  Move::tooclose_flag;//1 if the too close atoms have been loaded, 0 otherwise
int   Move::ntooclose;//number of atoms not satisfying the cutoffs
int   Move::swap;//boolean indicator, whether there can be swaps
int   Move::tot_moved_atoms;//total number of moved atoms (no swap:nmoved, swap:nmoved+1);
int   Move::tot_moved_virtuals;//total number of virtual atoms, whose position was changed by the move 
int	  Move::nswap_pairs;//number of mixed partials, where swap can occure
int  *Move::swap_type1;//pointer to the array containing the type of the first atom of the allowed swap pairs
int  *Move::swap_type2;//pointer to the array containing the type of the second atom of the allowed swap pairs
int  *Move::tclindices;//array of indices of atoms not satisfying the cutoffs
int  *Move::tclpaircount;//how many too close pair the given atom is involved
double  Move::swap_fraction;//fraction of swaps
double *Move::moveamp;//array of move amplitudes (in Angstrom)
double *Move::invmove_binwidth;//array of inverse bin width for the moved distance histograms /type

bool Move::nomove = false;//no atomic move is performed in this step
bool Move::makeE0shift;//indicator whether to make E0 shift instead of atomic move
bool Move::makemucorr;//indicator whether to make changes to the mu correction paramter for I(Q) fitting
bool Move::makefprimeshift;//indicator whether to make changes to the f' for I(Q) fitting

#ifdef _NO_PERIODIC
	double Move::R0_red;//reduced R0
#endif
//default constructor
Move::Move(SimpleCfg &conf)
	:config(conf)
{
	//initialising all pointers to 0
	tcl_ind=0;
	itooclose=0;
	tooclose_index=0;
	tooclose_remove=0;
	ntcl_pair=0;
	unscamp=0;//array of REDUCED move amplitudes
	indices=0;//indices of moved atoms in the cfg (start at 0)
	types=0;//types of atoms moved (start at 0)
	oldpos=0;//old coordinates of moved atoms
	newpos=0;//new coordinates of the moved atoms
	custparams=0;//No custom parameters
	tclpair_ind=0;//indices of too close pairs for the moved atom
	old_charge_c_pos=0;//old positions of the charge group centres
	charge_c_indices=0;//indices of the charge group centres of the moved atoms
#ifdef _NO_PERIODIC
	atom_binind_old=0;
#endif
};

//constructor for atomic RMC++

Move::Move(int &ntoocl, int *tclarray, SimpleCfg &conf)
	:config(conf) 
{	//this class defines the Move applied at each passage in the
	//main RMC loop 
	//some of the moves parameters appear in the .dat file
	//(e.g. move amplitude for each atom type)
	//if a non-standard (i.e non-RMCA move) is to be defined
	//then the possible additional parameters can be added
	//here 
	
	//arguments are:
	//1) ntoocl: the number of atoms not satisfying the cutoffs calculated by HistCalc (loaded too 
	//close atoms not included, they are represented by ntooclose
	//(this is initially needed for memory allocation for the tclindices array
	//but it is later updated as these atoms are moved and their number decreases)
	
	//2) tclarray is an array for all atoms containing how many too close pairs they are involved in calculated by HistCalc
	//3) for initializing the handle to the SimpleCfg object	

	//Check, whether the static members were initialised
	if (nmoved<1)//Check whether it was properly initialised
	{
		cout<<"\n"<<"*****ERROR*****"<<endl;
		cout<<"Move constructor for atomic move"<<endl;
		cout<<"The number of moved atoms is smaller than 1! "<<endl;
		cout<<"Cannot run this way, exiting..."<<endl;
		CleanExit();
	}
	if (ntypes<1)//Check whether it was properly initialised
	{
		cout<<"\n"<<"*****ERROR****"<<endl;
		cout<<"Move constructor for atomic move"<<endl;
		cout<<"The number of types is smaller than 1! "<<endl;
		cout<<"Cannot run this way, exiting..."<<endl;
		CleanExit();
	}
	if (moveamp==0)//Check whether it was properly initialised
	{
		cout<<"\n"<<"*****ERROR*****"<<endl;
		cout<<"Move constructor for atomic move"<<endl;
		cout<<"The move amplitude was not initialised! "<<endl;
		cout<<"Cannot run this way, exiting..."<<endl;
		CleanExit();
	}
	
	//constructor
	int i,k;

	max_pairs=0;
	
	//arrays of reduced move amplitudes - 1 per atom type
	SetArraysize(&unscamp,ntypes,"unscamp","Move::Move");
	
	//parameters updated at each application of the move
	//array dimensions depending on the number of moved atoms
	
	//indices of the moved atom(s) among the too close atoms, if it was a "too close" one, -1 otherwise
	SetArraysize(&itooclose,tot_moved_atoms,"itooclose","Move::Move");
	SetArraysize(&tooclose_index,tot_moved_atoms,"tooclose_index","Move::Move");//indices of the moved too close atom(s) in the configuration (starting with 0)
	SetArraysize(&tooclose_remove,tot_moved_atoms,"tooclose_remove","Move::Move");//indicator, whether the moved too close atom(s) can be removed from the list
	SetArraysize(&ntcl_pair,tot_moved_atoms,"ntcl_pair","Move::Move");//number of too close pairs the given moved tooclose atom(s) is/are involved in, which can be removed from the list 
	SetArraysize(&tcl_ind,tot_moved_atoms,"tcl_ind","Move::Move");//array, the serial number of this moved too close atoms among the moved too close atoms beginning 
							//with 0, -1 if the moved atom is not too close
	i = tot_moved_atoms;
	if (SimpleCfg::ntotvirtual > 0)
		i = tot_moved_atoms * (SimpleCfg::max_diff_virtual + 1);//the indices of the virtual sites the moved atom(s) involved in has to be szored as wells
	//the values for the virtuals will follow the last moved atom
	SetArraysize(&indices, i, "indices", "Move::Move");//indices of moved atoms in the cfg (start at 0)
	SetArraysize(&types, i, "types", "Move::Move");//types of atoms moved (start at 0)
	SetArraysize(&oldpos,i*3,"oldpos","Move::Move");//old coordinates of moved atoms
	SetArraysize(&newpos,i*3,"newpos","Move::Move");//new coordinates of the moved atoms
	
#ifdef _NO_PERIODIC
	SetArraysize(&atom_binind_old,tot_moved_atoms,"atom_binind_old","Move::Move");//old bin index of moved atoms
#endif
	if((RunParams::potential==1) || (RunParams::fnc==4))
	{
	  SetArraysize(&charge_c_indices,i,"charge_c_indices","Move::Move");//charge group centre indices of the moved atoms
	  SetArraysize(&old_charge_c_pos,i*3,"old_charge_c_pos","Move::Move");//old charge centre coordinates for the moved atoms
	}
	else
	{
	  charge_c_indices=0;
	  old_charge_c_pos=0;
	}
	if (RunParams::potential==1 && RunParams::fnc==4)
		update_charge_c=1;
	else
		update_charge_c=0;
	
	//the order of the coordinates (in box coordinates system) is
	//{x0,y0,z0,x1,y1,z1,x2,...,x(tot_moved_atoms-1),y(tot_moved_atoms-1),z(tot_moved_atoms-1)}
	
	//parameters linked to the custom move
	nparams=0;//no custom parameters
	custparams=0;//No custom parameters
		
	//initialisation of permanent arrays from the RunParams in argument
	for(i=0;i<ntypes;i++)
	{
		*(unscamp+i)=*(moveamp+i)/RunParams::boxedge;//reduced move amplitude
	}
		
	//Setting the parameters for the moveout option	
	if (ntooclose+ntoocl==0 && moveout==1)//if all atoms satisfy the cutoffs
	{
		cout << "\nNOTE(" << ++note << "): All atoms satisfy the cutoffs. Moveout option switched off. "<<endl;
		moveout=0;	//switch off the moveout option
		RunParams::moveout=0;// in the run object as well
	}

	//creation of the tclindices and tclpaircount arrays, if the atoms below cutoff were not loaded
	if (tooclose_flag==0 && moveout)
	{
		//the too close atoms (if there is any) were not loaded, initialize the connected parameters
		ntooclose=ntoocl;//number of atoms not satisfying the cutoffs
		
		if (ntooclose!=0)
		{
			SetArraysize(&tclindices,ntooclose,"tclindices","Move::Move");//array of indices of 'bad' atoms
			SetArraysize(&tclpaircount,ntooclose,"tclpaircount","Move::Move");//array of too close pairs the given atom is involved
		}
	

		i=0;//there were no loaded atoms below cutoff
		//Finding the atoms not satisfying the cutoffs to build the tclindices and tclpaircount arrays
		for (k=0;k<SimpleCfg::ntotal;k++)//for all atoms
		{
			if (tclarray[k]>0)
			{ 
				tclindices[i]=k;//put its index in the tclindices array
				tclpaircount[i]=tclarray[k];//put its atom pairs count in the tclpaircount array
				i++;
			}
		}	
	
	}
	if (moveout)
	{
		for (i=0;i<ntooclose;i++)
			if (tclpaircount[i]>max_pairs)
				max_pairs=tclpaircount[i];//determining the maximum number of tooclose pairs for the tooclose atoms
	}
	SetArraysize(&tclpair_ind,max_pairs*tot_moved_atoms,"tclpair_ind","Move::Move");//creating the array for the indices of the removable pairs of the moved atoms
											//they will be in the following order: indices for the first moved tooclose atom, 
										//indices for the second moved tooclose atom... 
		
	if (::debug)
		cout<<"Move object is created for atomic move"<<endl;
	
};	

//-------------------------------------------------------------------------
//Sets the static members
void Move::SetMoveParams()//Sets the static members
{
	int i;

	nmoved=RunParams::nmoved;
	ntypes=RunParams::ntypes;
	npartials=ntypes*(ntypes+1)/2;
	moveout=RunParams::moveout;
	swap_fraction=RunParams::swap_fraction;
	nswap_pairs=RunParams::nswap_pairs;
	swap_type1=RunParams::swap_type1;
	swap_type2=RunParams::swap_type2;

	if (swap_fraction>0)
		swap=1;//there can be swaps
	else
		swap=0;//no swap

	if (nmoved<1)//Check whether it was properly initialised
	{
		cout<<"\n"<<"*****ERROR*****"<<endl;
		cout<<"Move::GetMoveParams - The number of moved atoms is smaller than 1! "<<endl;
		cout<<"Cannot run this way, exiting..."<<endl;
		CleanExit();
	}
	
	if (ntypes<1)//Check whether it was properly initialised
	{
		cout<<"\n*****ERROR*****"<<endl;
		cout<<"Move::GetMoveParams - The number of types is smaller than 1! "<<endl;
		cout<<"Cannot run this way, exiting..."<<endl;
		CleanExit();
	}
		
	//array dimensions depending on the number of moved atoms
	if (swap)
		tot_moved_atoms=nmoved+1;//to have place for the extra atom the (nmoved-1)-th is swapped with
	else
		tot_moved_atoms=nmoved;//only nmoved atom
	SetArraysize(&moveamp,ntypes,"moveamp","Move::SetMoveParams");
	
	for (i=0;i<ntypes;i++)
		moveamp[i]=RunParams::pmaxmove[i];

#ifdef _NO_PERIODIC
	R0_red=RunParams::R0/RunParams::boxedge;//reduced R0
#endif
	
}

//Copy source object to dest object. If dest object was already properly initialised
//and dynamic arrays were allocated, then first the existing arrays are deleted, then
//new arrays are created
void Move::Copy(Move &dest)//Copy the Move object, which made the call to Move dest

{

	int i,size;

	dest.nparams=nparams;//number of parameters of a custom move
	dest.max_pairs=max_pairs;//the maximum of the too close pairs for an atom
	dest.config=config;//handle to the SimpleCfg object
	size = tot_moved_atoms;
	if (SimpleCfg::ntotvirtual > 0)
		size = tot_moved_atoms * (SimpleCfg::max_diff_virtual + 1);//the indices of the virtual sites the moved atom(s) involved in has to be szored as wells
	
	//copying tooclose_remove, the indicator, whether the too close atoms can be removed from the list
	if (dest.tooclose_remove!=0)
		delete [] dest.tooclose_remove;
	SetArraysize(&dest.tooclose_remove,tot_moved_atoms,"dest.tooclose_remove","Move::Copy");
	
	//copying tooclose_index, indices of the moved too close atoms in the configuration (starting with 0)
	if (dest.tooclose_index!=0)
		delete [] dest.tooclose_index;
	SetArraysize(&dest.tooclose_index,tot_moved_atoms,"dest.tooclose_index","Move::Copy");
	
	//copying ntcl_pair, number of too close pairs the given moved atom is involved in, which can be removed from the list 
	if (dest.ntcl_pair!=0)
		delete [] dest.ntcl_pair;
	SetArraysize(&dest.ntcl_pair,tot_moved_atoms,"dest.ntcl_pair","Move::Copy");
	
	//copying itooclose, indices of the moved atom(s) among the too close atoms, if it was a "too close" one, -1 otherwise
	if (dest.itooclose!=0)
		delete [] dest.itooclose;
	SetArraysize(&dest.itooclose,tot_moved_atoms,"dest.itooclose","Move::Copy");
	
	//copying indices of moved atoms in the cfg (start at 0) and types
	if (dest.indices!=0)
		delete [] dest.indices;
	SetArraysize(&dest.indices,size,"dest.indices","Move::Copy");
	
	//copying type of the moved atoms start at 0)
    if (dest.types!=0)
		delete [] dest.types;
	SetArraysize(&dest.types,size,"dest.types","Move::Copy");
		
	//copying the indicator, whether the moved atom is a tooclose or not
	if (dest.tcl_ind!=0)
		delete [] dest.tcl_ind;
	SetArraysize(&dest.tcl_ind,tot_moved_atoms,"dest.tcl_ind","Move::Copy");
	
	for(i=0;i<tot_moved_atoms;i++)
	{
		*(dest.tooclose_remove+i)=*(tooclose_remove+i);
		*(dest.tooclose_index+i)=*(tooclose_index+i);
		*(dest.ntcl_pair+i)=*(ntcl_pair+i);
		*(dest.itooclose+i)=*(itooclose+i);
		
		*(dest.tcl_ind+i)=*(tcl_ind+i);
	}
	for (i = 0; i < size; i++)
	{
		*(dest.indices + i) = *(indices + i);
		*(dest.types + i) = *(types + i);
	}

	//copying old and new positions of the moved atoms
	if (dest.oldpos!=0)
		delete [] dest.oldpos;
	SetArraysize(&dest.oldpos,3*size,"dest.oldpos","Move::Copy");
		
	if (dest.newpos!=0)
		delete [] dest.newpos;
	SetArraysize(&dest.newpos,3*size,"dest.newpos","Move::Copy");

	
	for(i=0;i<3*size;i++)
	{
		*(dest.oldpos+i)=*(oldpos+i);
		*(dest.newpos+i)=*(newpos+i);
	}

#ifdef _NO_PERIODIC
	//copying old bin indices of the moved atoms
	if (dest.atom_binind_old!=0)
		delete [] dest.atom_binind_old;
	SetArraysize(&dest.atom_binind_old,tot_moved_atoms,"dest.atom_binind_old","Move::Copy");
	for(i=0;i<tot_moved_atoms;i++)
		*(dest.atom_binind_old+i)=*(atom_binind_old+i);
#endif

	if (update_charge_c!=0)//charge centre are only used, if non-bonded, not tabulated potential is used with molecules
	{
		dest.update_charge_c=update_charge_c;
		if (dest.old_charge_c_pos!=0)
			delete [] old_charge_c_pos;
		SetArraysize(&dest.old_charge_c_pos,3*size,"dest.old_charge_c_pos","Move::Copy");
		for(i=0;i<3*size;i++)
			*(dest.old_charge_c_pos+i)=*(old_charge_c_pos+i);
		if (dest.charge_c_indices!=0)
			delete [] charge_c_indices;
		SetArraysize(&dest.charge_c_indices,size,"dest.charge_c_indices","Move::Copy");
		for(i=0;i<size;i++)
			*(dest.charge_c_indices+i)=*(charge_c_indices+i);
	}
	else dest.update_charge_c=0;

	//copying arrays of reduced move amplitudes - 1 per atom type

	if (dest.unscamp!=0)
		delete [] dest.unscamp;
	SetArraysize(&dest.unscamp,ntypes,"dest.unscamp","Move::Copy");
	
	for(i=0;i<ntypes;i++)
		*(dest.unscamp+i)=*(unscamp+i);//reduced move amplitude

	//Copy custom parameters, if there is any
	if (dest.custparams!=0)
		delete [] dest.custparams;
	if (nparams!=0)
	{
		SetArraysize(&dest.custparams,nparams,"dest.custparams","Move::Copy");
		
		for (i=0;i<nparams;i++)
			*(dest.custparams+i)=*(custparams+i);
	}
	else
		dest.custparams=0;

	//copying the indices of too close pairs the given moved atom is involved in, which can be removed from the list 
	if (dest.tclpair_ind!=0)
		delete [] dest.tclpair_ind;
	SetArraysize(&dest.tclpair_ind,max_pairs*tot_moved_atoms,"dest.tclpair_ind","Move::Copy");
	
	for(i=0;i<max_pairs*tot_moved_atoms;i++)
		*(dest.tclpair_ind+i)=*(tclpair_ind+i);//reduced move amplitude

		
};
		
//-------------------------saving the too close atoms to .tca file----------------
void Move::SaveTooClose(const char *file_name)
{
	int i;
	ofstream file;
	if (strlen(file_name)!=0)
		mystrcpy(tempfilename, FILE_NAME_SIZE + 10, file_name);
	else
		mystrcpy(tempfilename, FILE_NAME_SIZE + 10, tcafilename);

	OpenFile(file,tempfilename,"Move::SaveTooClose",0);//open file, check, whether it was successfully opened

	file<<"Atoms below cutoff created by Move::SaveTooClose!"<<endl;
	
	file<<" "<<J20<<SimpleCfg::ntotal<<" number of atoms"<<endl;
	file<<" "<<J20<<ntypes<<" number of atom types"<<endl;
	file<<" "<<J20<<ntooclose<<" number of atoms below cutoff"<<endl;
	file.precision(8);
	file.setf(ios::fixed, ios::floatfield);
	file.setf(ios::right, ios::adjustfield);
	file<<"Cutoffs/type in A"<<endl;
	for (i=0;i<npartials;i++) 
		file<<" "<<RunParams::pcutoff[i]<<endl;
	file<<endl;
	file.precision(6);
	file.unsetf(ios::fixed);
	file.unsetf(ios::right);
	
	file<<"Index and bad pair count of the atoms below cutoff (index starts with zero!)"<<endl;
	for (i=0;i<ntooclose;i++) 
		file<<"\t"<<J10<<tclindices[i]<<"\t"<<J10<<tclpaircount[i]<<endl;

	file.close();
};
//------------------------loading the too close atoms from .tca file-----------------------
int Move::LoadTooClose()
{
	ifstream file;
	SafeOpenTextFile(file,tcafilename);
	if (CheckFileState(file,"Move::LoadTooClose",tcafilename)==0)
	{
		cout<<"There is no open file to load the atoms below cutoff from"<<endl;
		return(0);//the load was not successful, histogram has to be calculated
	}

	int i,dummy;
	double rdummy,diff;
	char *conv_numb=NULL,*conv_t;
	
	SetArraysize(&conv_t,100,"conv_t","Move::LoadTooClose");
		
	SkipLine(file,tcafilename,1);
	dummy=ReadThisLine(file,1,1,"total number of atoms","Move::LoadTooClose",tcafilename);
	if (dummy!=SimpleCfg::ntotal)
	{
		cout << "\nWARNING(" << ++warn << "): The number of atoms is "<<SimpleCfg::ntotal<<" in the .cfg and "<<dummy<<\
			" in the .tca file are not the same!"<<endl;
		cout << "\tLoading was not successful, histogram and coordination numbers will be calculated!" << endl;
		return(0);//load was not successful
	}
	dummy=ReadThisLine(file,1,1,"ntypes","Move::LoadTooClose",tcafilename);
	if (dummy!=ntypes)
	{
		cout << "\nWARNING(" << ++warn << "): The number of atom types is "<<ntypes<<" in the .cfg and "<<dummy<<\
			" in the .tca file are not the same!"<<endl;
		cout << "\tLoading was not successful, histogram and coordination numbers will be calculated!" << endl;
		return(0);//load was not successful
	}
	ntooclose=ReadThisLine(file,1,1,"ntooclose","Move::LoadTooClose",tcafilename);
	
	SkipLine(file,tcafilename,1);
	for (i=0;i<npartials;i++) 
	{
		IntToStr(&conv_numb,i+1);
		mystrcpy(conv_t,100,"cutoff for partial ");
		mystrcat(conv_t, 100, conv_numb);
		rdummy=ReadThisLine(file,1,1.0,conv_t,"Move::LoadTooClose",tcafilename);
		diff=fabs(rdummy-RunParams::pcutoff[i]);
		if (diff>1e-15)
		{
			cout.precision(16);
			cout << "\nWARNING(" << ++warn << "): The cutoff for the "<<i<<". partial is "<<RunParams::pcutoff[i]<<" in .dat file and "<<rdummy<<\
			" in the .tca file are not the same!"<<endl;
			cout << "\tLoading was not successful, histogram and coordination numbers will be calculated!" << endl;
			return(0);//load was not successful
			
		}

	}

	SkipLine(file,tcafilename,1);
	SkipLine(file,tcafilename,1);
	SetArraysize(&tclindices,ntooclose,"tclindices","Move::LoadTooClose");
	SetArraysize(&tclpaircount,ntooclose,"tclpaircount","Move::LoadTooClose");
	
	for (i=0;i<ntooclose;i++)
	{
		IntToStr(&conv_numb,i+1);
		mystrcpy(conv_t, 100, "tclindex for too close atom ");
		mystrcat(conv_t, 100, conv_numb);
		tclindices[i]=ReadThisLine(file,3,1,conv_t,"Move::LoadTooClose");
		mystrcpy(conv_t, 100, "tclpaircount for too close atom ");
		mystrcat(conv_t, 100, conv_numb);
		if (i<ntooclose-1)
			tclpaircount[i]=ReadThisLine(file,1,1,conv_t,"Move::LoadTooClose",tcafilename);
		else
			tclpaircount[i]=ReadThisLine(file,1,1,conv_t,"Move::LoadTooClose");
	}
	//Checking, whether loading was successful
	if (CheckReadFileState(file,"Move::LoadTooClose",tcafilename))
	{
		tooclose_flag=1;//to preserve, that the load was successful
		return(1);//load was successful
	}
	else
	{
		cout << "\nWARNING(" << ++warn << "): Loading was not successful, histogram and coordination numbers will be calculated!"<<endl;
		return(0);//loading was not successful
	}
	if (conv_numb != NULL)
		delete [] conv_numb;
	next_line_pos = -1;
	file.close();
	return(1);
};

//-----------resets the parameters connected to the too close atoms to zero-----------
void Move::ResetTooClose()
{
	//resetting to zero
	ntooclose=0;
	tooclose_flag=0;
	//deleting the arrays if they were created
	if (tclindices!=NULL)
		delete [] tclindices;

	if (tclpaircount!=NULL)
		delete [] tclpaircount;
};

//--------removing the other atom of the too close pair from the lists, if the distance is now above cutoff--------
//if the tooclose atom indicated by the tclpair_ind[iselected*max_pairs+i] have been removed for the previous moved atom of the 
//same move, if tot_moved_atoms>1, then nothing is done here 
void Move::RemoveTooClosePairs(int iselected)
{
	int i,index;
	for (i=0;i<ntcl_pair[iselected];i++)//cycling through all the neighbours of the moved too close atom, where now the distance
				//is above cutoff
	{
		for (index=0;index<ntooclose;index++)//cycling through the atoms of the too close list
		{

			if (tclindices[index]==tclpair_ind[iselected*max_pairs+i])
			{
				//index is the position of the neighbour atom in the tclindices and tclpaircount arrays
				tclpaircount[index]--;//decrease the bad pair count for this atom
				if (tclpaircount[index]==0)//this was the last bad pair for this atom, remove it from the list
				{
					if (index==ntooclose-1)//this is the last on the list
					{
						tclindices[ntooclose-1]=-1;//erase the index, to prevent to cause any problems
						ntooclose--;//decrease the number of too close atoms
					}
					else
					{
						//replace the index of this atom with the last one on the list
						tclindices[index]=tclindices[ntooclose-1];
						//replace the too close pair count of this atom with the last one on the list
						tclpaircount[index]=tclpaircount[ntooclose-1];
						tclindices[ntooclose-1]=-1;//erase the index, to prevent to cause any problems

						ntooclose--;//decrease the number of too close atoms
					}
				}
				break;//continue with next neighbour

			}
			
		}
	}
};

//reset the new positions in case the move is rejected
void Move::ResetCoord(FNC_POT *fnc)
{
	int i,imoved,*p_index;
	double  *p_coord_moved,*p_coord;
	double *p_chargegr_c=0;

	//copy back the coordinates
	p_coord_moved=oldpos;//sets the pointer to the old coordinates of the first moved atom
	if(fnc) p_chargegr_c=old_charge_c_pos;//sets the pointer to the old coordinates of the first moved atom~s charge group centre
	p_index=indices;//sets the pointer to the index of the first moved atom
	for (imoved=0;imoved<tot_moved_atoms+tot_moved_virtuals;imoved++)
	{
		//sets the pointer to the x coordinate of the first moved atom
		p_coord=config.positions+3*(*p_index);
		for (i=0;i<3;i++)
			*p_coord++=*p_coord_moved++;//copying the moved coordinates
		if (fnc && update_charge_c)
		{
			p_coord=config.charge_gr_centre+3*fnc->charge_centre[*p_index];
			for (i=0;i<3;i++)
				*p_coord++=*p_chargegr_c++;//copying the moved coordinates
		}
		p_index++;
	}
};

#ifdef _NO_PERIODIC
	//restoring the original central nin indices, if the move was not accepted
	void Move::ResetBinInd()
	{
		int imoved,*p_index,*p_old;
		p_index=indices;//sets the pointer to the index of the first moved atom
		p_old=atom_binind_old;
		for (imoved=0;imoved<tot_moved_atoms;imoved++)
		{
			HistoSet::atom_binind[*p_index]=*p_old;
			p_old++;
			p_index++;
		}
	};
#endif

